#ifndef CLATTICEPROTEINPIVOTMOVESTATE_HH_
#define CLATTICEPROTEINPIVOTMOVESTATE_HH_


#include "CState.hh"
#include "LatticeProteinPivotMoveState.hh"

namespace ell
{	
		/**
		 * An object of this class represents a lattice protein pivot move
		 * state in a compressed representation.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class CLatticeProteinPivotMoveState : public CState
	{
	private:
			//! the length of the uncompressed lattice protein sequence
		size_t seqLength;

			/*! the lattice model this lattice protein belongs to */
		const LatticeModel*	lattice;		
		
			/*! the contact energy function */
		const ContactEnergyFunction*	energy;	

			//! the lattice protein sequence in compressed representation
		std::vector<unsigned char> cSequence;

	protected:
			//! the lattice protein structure as relative move sequence
			//! in compressed representation
		std::vector<unsigned char> cStructure;

		CLatticeProteinPivotMoveState()
		{}

			/*! Returns the uncompressed lattice protein sequence string.
			 */
		virtual std::string getUncompressedSeqStr() const;

			/*! Returns the uncompressed lattice protein structure string
			 *  (relative moves).
			 */
		virtual std::string getUncompressedStructStr() const;

	public:
		CLatticeProteinPivotMoveState(const LatticeModel* _lattice, 
									  const ContactEnergyFunction* _energy, 
									  const Sequence& seq,
									  const Structure& struc);

		virtual ~CLatticeProteinPivotMoveState();

	// abstract functions (CState)
		virtual bool			operator== (const CState& cState2) const;
		virtual bool			operator!= (const CState& cState2) const;

		virtual CLatticeProteinPivotMoveState*	clone() const;

		virtual std::string	getCStateStringRepresentation() const;

			/*! Returns a specific std::string representation of the
			 *  uncompressed representation of this state.
			 * 
			 *  RELATIVEMOVESTRING(SEQUENCE) */
		virtual std::string	getUncomprStateStringRepresentation() const;

		virtual State* uncompressedClone() const;

		virtual void	uncompress(State& state) const;

		virtual CLatticeProteinPivotMoveState*
				  readBinaryInputClone(std::ifstream& source) const;

		virtual void	writeBinaryOutput(std::ofstream& destination) const;
	};

}

#endif /*CLATTICEPROTEINPIVOTMOVESTATE_HH_*/
