#include <ell/CLatticeProteinPivotMoveStructState.hh>
#include <fstream>	// for readBinaryInputClone(..), writeBinaryOutput(..)

namespace ell
{
	CLatticeProteinPivotMoveStructState::CLatticeProteinPivotMoveStructState(
	 const Structure& struc) {
	cStructure = compressSeq(struc,
							 LatticeProteinPivotMoveStructState::
							  getCommonLatticeModel()->getDescriptor()
							  ->getAlphabet());
	}

	CLatticeProteinPivotMoveStructState::
	~CLatticeProteinPivotMoveStructState() {
	}

	std::string
	CLatticeProteinPivotMoveStructState::getUncompressedSeqStr() const {
		return LatticeProteinPivotMoveStructState::getCommonSequenceString();
	}

	std::string
	CLatticeProteinPivotMoveStructState::getUncompressedStructStr() const {
			// structure string holds one character less than sequence string
		return uncompressSeq(cStructure,
							  LatticeProteinPivotMoveStructState::
							   getCommonLatticeModel()->getDescriptor()
							   ->getAlphabet(),
							  LatticeProteinPivotMoveStructState::
							   getCommonSequenceString().size()-1);
	}

	bool	
	CLatticeProteinPivotMoveStructState::operator== (const CState& _cState2) const {
		CLatticeProteinPivotMoveStructState cState2 =
		 dynamic_cast<const CLatticeProteinPivotMoveStructState&>(_cState2);
		return (this->cStructure == cState2.cStructure);
	}

	bool
	CLatticeProteinPivotMoveStructState::operator!= (const CState& _cState2) const {
		CLatticeProteinPivotMoveStructState cState2 =
		 dynamic_cast<const CLatticeProteinPivotMoveStructState&>(_cState2);
		return (this->cStructure != cState2.cStructure);
	}

	CLatticeProteinPivotMoveStructState* 
	CLatticeProteinPivotMoveStructState::clone() const {
		return new CLatticeProteinPivotMoveStructState(*this);
	}

	std::string
	CLatticeProteinPivotMoveStructState::getCStateStringRepresentation() const {
		std::string strRepr;
		for (size_t i = 0; i < cStructure.size(); i++) {
			strRepr += cStructure[i];
		}
		return strRepr;
	}

	std::string
	CLatticeProteinPivotMoveStructState::getUncomprStateStringRepresentation()
	 const {
		return getUncompressedStructStr();
	}

	State*
	CLatticeProteinPivotMoveStructState::uncompressedClone() const {
		return new LatticeProteinPivotMoveStructState(
					 getUncompressedStructStr(), false);
	}

	void
	CLatticeProteinPivotMoveStructState::uncompress(State& state) const {
		dynamic_cast<LatticeProteinPivotMoveStructState&>(state) =
		 LatticeProteinPivotMoveStructState(getUncompressedStructStr(), false);
	}

	CLatticeProteinPivotMoveStructState*
	CLatticeProteinPivotMoveStructState::readBinaryInputClone(std::ifstream& source)
	 const {

			// create new CLatticeProteinPivotMoveStructState
		CLatticeProteinPivotMoveStructState* cState =
		 new CLatticeProteinPivotMoveStructState();

		size_t size;

			// read cStructure
		source.read(reinterpret_cast<char*>(&size), sizeof(size));
		cState->cStructure.resize(size);
		for (size_t i = 0; i < cState->cStructure.size(); i++) {
			source.read(reinterpret_cast<char*>(&(cState->cStructure[i])),
						  sizeof(cState->cStructure[i]));
		}

		return cState;
	}

	void
	CLatticeProteinPivotMoveStructState::writeBinaryOutput(std::ofstream& destination)
	 const {
			// write cStructure
		size_t size = cStructure.size();
		destination.write(reinterpret_cast<const char*>(&size), sizeof(size));
		for (size_t i = 0; i < size; i++) {
			destination.write(reinterpret_cast<const char*>(&(cStructure[i])),
							   sizeof(cStructure[i]));
		}
	}

} // namespace ell
