#ifndef LATTICEDESCRIPTORCUB_HH_
#define LATTICEDESCRIPTORCUB_HH_


#include "LatticeDescriptor.hh"

namespace ell
{

		/**
		 * Lattice property handler for the integer cubic lattice.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class LatticeDescriptorCUB : public LatticeDescriptor
	{
	protected:

			// abstract function implementation
		virtual unsigned int getNeighborDataSize() const;
		virtual const NeighborData *getNeighborData() const;
		virtual unsigned int getAutomorphismDataSize() const;
		virtual const AutomorphismData *getAutomorphismData() const;
	public:
	
		LatticeDescriptorCUB();
		LatticeDescriptorCUB(const LatticeDescriptorCUB& toCopy);
		virtual ~LatticeDescriptorCUB();
	};

}


namespace ell
{
		/*! 
		 * A specialized LatticeNeighborhood to handle the
		 * the NeighborVector objects of the cubic lattice and the 
		 * access to them.
		 */
	class LatticeNeighborhoodCUB : public LatticeNeighborhood
	{
	public:
		LatticeNeighborhoodCUB(	const MoveAlphabet* moveAlph, 
								const NeighSet& neighbors)
		 :	LatticeNeighborhood(moveAlph, neighbors) 
		{
		}
		
		LatticeNeighborhoodCUB(	const LatticeNeighborhood& nh ) 
		:	LatticeNeighborhood( nh )
		{
		}
		
								
		virtual ~LatticeNeighborhoodCUB()
		{}
			//! Returns whether or not a vector v belongs to the neighborhood.
		bool isElement( const IntPoint& v) const {
			return (v.getX()* v.getX() + v.getY()*v.getY() + v.getZ()*v.getZ()) 
					== 1;
/*
			return (abs( v.getX() + v.getY() + v.getZ() ) == 1) && 
			((abs(v.getX()) == 1 && v.getY() == 0 && v.getZ() == 0) ||
			(v.getX() == 0 && 
				((abs(v.getY()) == 1 && v.getZ() == 0) ||
				 (v.getY() == 0 && abs(v.getZ()) == 1))) 
			);
*/
		}
	};

} // namespace ell


#endif /*LATTICEDESCRIPTORCUB_HH_*/
